from datetime import datetime
import re
import time
import json
import jwt

import requests

from .const import API_KEY_AUTHENTICATION, API_KEY_CUSTOMER_SERVICES, API_KEY_USER_ACCOUNTS, REST_API_URL
from .Entities import AccountInfo, Service

class WMClient:
    def __init__(
        self, email, password
    ):
        self.email = email
        self.password = password
        self._session_token = None
        self._access_token = None
        self._refresh_token = None
        self._id_token = None
        self._user_id = None
        self._token_expires_time = None
        self._okta_access_token = None
        self._client_id = None
        self._issuer = None

    def _string_escape(self, input : str, encoding='utf-8'):
        return input.encode('latin1').decode('unicode-escape').encode('latin1').decode(encoding)

    def authenticate(self):
        self._apiKey = API_KEY_AUTHENTICATION
        data = self.api_post("user/authenticate", {"username": self.email, "password": self.password, "locale": "en_US"})
        response_data = data["data"]

        self._session_token = response_data["sessionToken"]
        self._access_token = response_data["access_token"]
        self._refresh_token = response_data["refresh_token"]
        self._id_token = response_data["id_token"]
        self._user_id = response_data["id"]
        self._token_expires_time = time.time() + response_data["expires_in"]
        decoded_jwt = jwt.decode(response_data["access_token"], options={"verify_signature": False})
        self._client_id = decoded_jwt["cid"]
        self._issuer = decoded_jwt["iss"]
        return data


    def okta_authorize(self):
        # get from access token issuer
        response = requests.get(self._issuer+"/v1/authorize", {"client_id": self._client_id, 
        "nonce": "x", "prompt": "none", 
        "response_mode": "okta_post_message", "response_type": "token", "state": "x", 
        "scope": "openid email offline_access", "redirect_uri": "https://www.wm.com", "sessionToken": self._session_token})
        response.raise_for_status()
        result = re.search("access_token\s*=\s*'(.+?)'", response.text, re.MULTILINE)
        self._okta_access_token = self._string_escape(result.group(1))   

    def get_accounts(self):
        self._apiKey = API_KEY_USER_ACCOUNTS
        
        response = requests.get(REST_API_URL + f"authorize/user/{self._user_id}/accounts", headers=self.headers, params={"timestamp": time.time()*1000, "lang":"en_US"})
        response.raise_for_status()
      
        jsonData = json.loads(response.content.decode("UTF-8"))

        results = []
        for acct in jsonData["data"]["linkedAccounts"]:
            results.append(AccountInfo(acct))
        return results

    def get_services(self, account_id):
        self._apiKey = API_KEY_CUSTOMER_SERVICES

        response = requests.get(REST_API_URL + f"account/{account_id}/services", headers = self.headers, params={"lang": "en_US","serviceChangeEligibility": "Y", "userId": self._user_id})
        response.raise_for_status()

        jsonData = json.loads(response.content.decode("UTF-8"))
        results = []
        for svc in jsonData["services"]:
            results.append(Service(svc))

        return results

    def get_service_pickup(self, account_id, service_id):
        self._apiKey = API_KEY_CUSTOMER_SERVICES

        response = requests.get(REST_API_URL + f"account/{account_id}/service/{service_id}/pickupinfo", headers=self.headers, params={"lang": "en_US", "checkAlerts":"Y", "userId": self._user_id})
        response.raise_for_status()

        jsonData = json.loads(response.content.decode("UTF-8"))

        pickupDates = []
        for dateStr in jsonData["pickupScheduleInfo"]["pickupDates"]:
            date = datetime.strptime(dateStr, "%m-%d-%Y")
            pickupDates.append(date)

        return pickupDates       

    def api_get(self, path="", query=None):
        response = requests.get(REST_API_URL + path)
        response.raise_for_status()
        return json.loads(response.content.decode("UTF-8"))

    def api_post(self, path="", data=None):
        response = requests.post(REST_API_URL + path, headers=self.headers, json=data)
        response.raise_for_status()

        return json.loads(response.content.decode("UTF-8"))


    @property
    def headers(self):
        headers = {"Content-Type": "application/json", "apiKey": self._apiKey}

        if self._okta_access_token is not None:
            headers["oktaToken"] = self._okta_access_token

        return headers



REST_API_URL = "https://rest-api.wm.com/"

API_KEY_PAYMENT= "6ACAFBEEC54039F21716"
API_KEY_ROLLOFF= "5B1B5D23674B426465F0"
API_KEY_REPAIR= "6277FF29BB19666078AC"
API_KEY_AUTHENTICATION= "6277FF29BB19666078AC"
API_KEY_BALANCE_INVOICE= "615F2EFC82FF8BB2F864"
API_KEY_CUSTOMER_SERVICES= "1611D2AB8045EFB4D148"
API_KEY_USER_ACCOUNTS= "AABFF96D4542575160FC"
API_KEY_USER_REQUEST_SERVICE= "F1ADA754F65E546BFC97"
API_KEY_CHAT_LOGGED_IN= "2DC6C5B8146A1C281F96"
API_KEY_CHAT_LOGGED_IN_MOBILE= "306B8BA80546A3CE5DEC"
API_KEY_APPLE_PAY= "B8F7CE5A093141B1FFA2"
API_KEY_APPLE_PAY_SCHEDULED= "4E6D3BBDE553BE2807ED"
API_KEY_APPLE_PAY_AUTOPAY= "6CBA95B0A31A94C3C91C"
API_KEY_APPLE_PAY_PROCESS= "0A4031783CB69A4BC435"
API_KEY_BULK_AUTOPAY_PAPERLESS_ENROLL= "65D55CF6F9D38F50E02C"
API_KEY_SERVICE_CHANGE= "74DC8D4A7C565E51908E"
API_KEY_PAUSE_SERVICE= "74DC8D4A7C565E51908E"
API_KEY_ALERTS= "C861FBA520CDD18ADC8D"
API_KEY_INCIDENTS_BY_USER_ID= "6277FF29BB19666078AC"
API_KEY_EXTRA_PICKUP= "615F2EFC82FF8BB2F864"
API_KEY_BULK_PICKUP= "6277FF29BB19666078AC"
API_KEY_RECYCLING_PICKUP= "615F2EFC82FF8BB2F864" 
API_KEY_HISTORY_VISUALS= "615F2EFC82FF8BB2F864"
API_KEY_PENDING_CHARGES= "615F2EFC82FF8BB2F864"
API_KEY_COMBINED_ORDER_HISTORY= "6277FF29BB19666078AC"
API_KEY_SURVEY= "6277FF29BB19666078AC"
API_KEY_HOLIDAYS_USER_BY_CUSTOMER_ID= "6277FF29BB19666078AC"
API_KEY_HOLIDAYS_USER_BY_ADDRESS= "6277FF29BB19666078AC"
API_KEY_CONTACT_INFO_USER_BY_CUSTOMER_ID= "6277FF29BB19666078AC"
API_KEY_CONTACT_INFO_USER_BY_ADDRESS= "6277FF29BB19666078AC"
API_KEY_USER_BILLING_DETAILS= "6277FF29BB19666078AC"
API_KEY_PAPERLESS_PREFERENCES= "6277FF29BB19666078AC"
API_KEY_TEXT_FOR_BULK= "65D55CF6F9D38F50E02C"
API_KEY_TEXT_FOR_ROLLOFF= "65D55CF6F9D38F50E02C"
API_KEY_TEXT_FOR_XPU= "65D55CF6F9D38F50E02C"
API_KEY_TEXT_FOR_CONTAINER_REPAIR= "65D55CF6F9D38F50E02C"
API_KEY_REPORT_MISSED_PICKUP= "6277FF29BB19666078AC"
API_KEY_TEXT_FOR_CITY_BILLED_BULK= "65D55CF6F9D38F50E02C"
API_KEY_REFRESH_TOKEN= "0zqoBUUwHpGwp04xxkwM5zKBCy8SeLYm9BibvwEgN2g"
API_KEY_ORDERS= "E5E2902D4E578F37CNQ3"


class AccountInfo():
    def __init__(self, json):
        address = json["serviceAddress"]
        self.name = f'{address["street"]} {address["city"]} {address["state"]}'
        self.id = json["custAccountId"]

class Service():
    def __init__(self, json):
        self.id = json["serviceId"]
        self.name = json["serviceDescription"]


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="waste_management",
    version="1.0",
    author="Dominick Meglio",
    license="MIT",
    author_email="dmeglio@gmail.com",
    description="Provides ability to query the Waste Management REST API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dcmeglio/waste_management",
    packages=setuptools.find_packages(),
    install_requires=["requests", "pyjwt"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import sys
import argparse

from . import WMClient

# create parser for arguments
parser = argparse.ArgumentParser(
    usage="python -m petsafe email pasword"
)
parser.add_argument("email", help="account email address")
parser.add_argument("password", help="account password")

# if no arguments specified, show help
if len(sys.argv) < 3:
    parser.print_help()
    sys.exit(1)

# parse for arguments
args = parser.parse_args()

client = WMClient(args.email, args.password)
client.authenticate()
client.okta_authorize()

accounts = client.get_accounts()

for account in accounts:
    print(account.name)
    services = client.get_services(account.id)

    for svc in services:
        print(svc.name)
        print(client.get_service_pickup(account.id, svc.id))